/*
* Copyright (c) 2005-2010 Nokia Corporation and/or its subsidiary(-ies). 
* All rights reserved.
* This component and the accompanying materials are made available
* under the terms of "Eclipse Public License v1.0"
* which accompanies this distribution, and is available
* at the URL "http://www.eclipse.org/legal/epl-v10.html".
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  CSysApBtSapController implementation
 *
*/


// INCLUDES

#include "SysApBtSapControllerImpl.h"
#include "SysApSubscriber.h"
#include "SysApAppUi.h"
#include <BTSapDomainPSKeys.h>
#include <btengdevman.h>
#include <btengconnman.h>
#include <StringLoader.h>
#include <coemain.h>
#include <SysAp.rsg>
#include <startupdomainpskeys.h>


MSysApBtSapController* CreateSysApBtSapControllerL( CSysApAppUi& aSysApAppUi )
    {
    return CSysApBtSapController::NewL( aSysApAppUi );
    }

// ============================ LOCAL FUNCTIONS ==============================
// -----------------------------------------------------------------------------
// SysApCBTDeviceArrayCleanup
// Cleanup operation for CBTDeviceArray
// -----------------------------------------------------------------------------
static void SysApCBTDeviceArrayCleanup( TAny* aItem )
    {
    CBTDeviceArray* devices = static_cast<CBTDeviceArray*>( aItem );
    devices->ResetAndDestroy();
    delete devices;
    }

// ============================ MEMBER FUNCTIONS ==============================

// ----------------------------------------------------------------------------
// CSysApBtSapController::NewL() 
// ----------------------------------------------------------------------------
CSysApBtSapController* CSysApBtSapController::NewL( CSysApAppUi& aSysApAppUi )
    {
     //
    CSysApBtSapController* self = new(ELeave) CSysApBtSapController( aSysApAppUi );
    CleanupStack::PushL( self );
    self->ConstructL();
    CleanupStack::Pop();
    return self;
    }

// ----------------------------------------------------------------------------
// CSysApBtSapController::CSysApBtSapController() 
// ----------------------------------------------------------------------------
CSysApBtSapController::CSysApBtSapController( CSysApAppUi& aSysApAppUi ) :
    iSysApAppUi( aSysApAppUi ),
    iBtSapEnabled( EFalse ),
    iSwitchingToOffline( EFalse )
    {
    }

// ----------------------------------------------------------------------------
// CSysApBtSapController::ConstructL() 
// ----------------------------------------------------------------------------
void CSysApBtSapController::ConstructL()
    {
     //
    iBtSapSubscriber = CSysApSubscriber::NewL( *this, KPSUidBluetoothSapConnectionState, KBTSapConnectionState );
    iBtSapSubscriber->Subscribe();
    }

// ----------------------------------------------------------------------------
// CSysApBtSapController::~CSysApBtSapController() 
// ----------------------------------------------------------------------------

CSysApBtSapController::~CSysApBtSapController()
    {
     //
    delete iBtSapSubscriber;
    }

// ----------------------------------------------------------------------------
// CSysApBtSapController::BtSapEnabled()
// ----------------------------------------------------------------------------
TBool CSysApBtSapController::BtSapEnabled()
    {
    return iBtSapEnabled;
    }

// ----------------------------------------------------------------------------
// CSysApBtSapController::Disconnect()
// ----------------------------------------------------------------------------
TInt CSysApBtSapController::Disconnect()
    {
     //

    RBTDevAddrArray addrArray;
    CBTEngConnMan* btConnMan = NULL;
    TRAPD( err, btConnMan = CBTEngConnMan::NewL() );
    if ( err == KErrNone )
        {
        err = btConnMan->GetConnectedAddresses( addrArray, EBTProfileSAP );
        if ( (err == KErrNone) && addrArray.Count() )
            {
            err = btConnMan->Disconnect( addrArray[0], EBTDiscGraceful );
            }
        else
            {
             //
            }            
        }
    else
        {
         //
        }        
    delete btConnMan;
    addrArray.Close();
    
    return err;
    }

// ----------------------------------------------------------------------------
// CSysApBtSapController::SwitchingToOffline()
// ----------------------------------------------------------------------------
void CSysApBtSapController::SwitchingToOffline()
    {
     //
    iSwitchingToOffline = ETrue;
    }
    
// ----------------------------------------------------------------------------
// CSysApBtSapController::SimApplicationsClosed()
// ----------------------------------------------------------------------------
void CSysApBtSapController::SimApplicationsClosed()
    {
     //
    TInt err ( 0 );

    err = iSysApAppUi.SetStarterState( RStarterSession::EBTSap );

    if ( err )
        {
         //
        // We should end up in this branch only in the early integration phase of a product, and the
        // device is most likely dysfunctional after this error.
        }
    else
        {
        iFinalizeDisconnectNeeded = ETrue;
        
        // No RF available so signal bars should be same as in offline mode.
        iSysApAppUi.IgnoreZeroNetworkBarNotifications(ETrue);
        TRAP( err, iSysApAppUi.UpdateSignalBarsL( KAknSignalOffLineMode ) );

        iSysApAppUi.SetNetworkConnectionAllowed( ECoreAppUIsNetworkConnectionNotAllowed );

        // The SAP connection is accepted via P&S key
        RProperty::Set( KPSUidBluetoothSapConnectionState, KBTSapConnectionState, EBTSapAccepted );
        }
        
     //
    }

// ----------------------------------------------------------------------------
// CSysApBtSapController::FinalizeDisconnect()
// ----------------------------------------------------------------------------
void CSysApBtSapController::FinalizeDisconnect()
    {
     //
    TInt err ( 0 );
    iFinalizeDisconnectNeeded = EFalse;
    iSysApAppUi.IgnoreZeroNetworkBarNotifications(EFalse);

    if ( iSysApAppUi.ActiveProfileId() == KOfflineModeProfileId )
        {
        if ( iSwitchingToOffline )
            {
            TRAP( err, iSysApAppUi.SwitchFromOnlineToOfflineModeL() );
             //
            iSwitchingToOffline = EFalse;
            }
        else
            {
            err = iSysApAppUi.SetStarterState( RStarterSession::EOffline );
             //
            }
        }
    else
        {
        err = iSysApAppUi.SetStarterState( RStarterSession::ENormal );
        
        if ( err == KErrNone )
            {
            iSysApAppUi.SetNetworkConnectionAllowed(ECoreAppUIsNetworkConnectionAllowed);
            }
        
         //
        }
    }

// ----------------------------------------------------------------------------
// CSysApBtSapController::HandlePropertyChangedL()
// ----------------------------------------------------------------------------
void CSysApBtSapController::HandlePropertyChangedL( const TUid& aCategory, const TUint aKey )
    {
    TInt simStatus = ESimStatusUninitialized;
    if ( aCategory == KPSUidBluetoothSapConnectionState && aKey == KBTSapConnectionState )
        {
        TInt value( 0 );
        RProperty::Get( aCategory, aKey, value );
        switch ( value )
            {
            case EBTSapNotConnected:
                 //
                if ( iFinalizeDisconnectNeeded )
                    {
                    FinalizeDisconnect();
                    }
                iBtSapEnabled = EFalse;
                break;
            case EBTSapConnecting:
                 //
                //Ensure SIM is present
                simStatus = iSysApAppUi.StateOfProperty( KPSUidStartup, KPSSimStatus );
                if ( !iBtSapEnabled && !(simStatus == ESimNotPresent || simStatus == ESimNotSupported))
                    {
                    iSysApAppUi.InitCloseSimApplicationsL();
                    }
                break;
            case EBTSapConnected:
                 //
                iBtSapEnabled = ETrue;
                break;
            default:
                 //
                break;
            }
        }
    }


// ----------------------------------------------------------------------------
// CSysApBtSapController::GetBtSapDeviceNameL()
// ----------------------------------------------------------------------------
HBufC* CSysApBtSapController::GetBtSapDeviceNameL()
    {
     //
    
    TBTDevAddr address;
    RBTDevAddrArray addrArray;
    CBTEngConnMan* btConnMan = NULL;
    TRAPD( err, btConnMan = CBTEngConnMan::NewL() );
    CleanupStack::PushL( btConnMan );
    if ( err == KErrNone )
        {
        err = btConnMan->GetConnectedAddresses( addrArray, EBTProfileSAP );
        if ( ( err == KErrNone ) && addrArray.Count() )
            {
            address = addrArray[0];
            }
        else if ( ( err == KErrNone) && !addrArray.Count() ) // not known whether this will ever occur
            {
             //
            err = KErrNotFound; // discontinue name fetching
            }
        else
            {
             //
            }            
        }
    else
        {
         //
        }        
    CleanupStack::PopAndDestroy( btConnMan );
    addrArray.Close();

    HBufC* prompt = NULL;

    if ( err == KErrNone )            
        {
        CBTEngDevMan* devMan = CBTEngDevMan::NewLC( NULL );
            
        TBTRegistrySearch criteria;
        criteria.FindAddress( address );
            
        CBTDeviceArray* devices = new (ELeave) CBTDeviceArray( 1 );
        CleanupStack::PushL( TCleanupItem( SysApCBTDeviceArrayCleanup, devices ) );
            
        err = devMan->GetDevices( criteria, devices );
            
        if ( err != KErrNone )
            {
             //
            }
        else
            {
            if ( devices->At( 0 )->IsValidFriendlyName() )
                {
                prompt = devices->At( 0 )->FriendlyName().AllocL();
                }
            else if ( devices->At( 0 )->IsValidDeviceName() )
                {
                prompt = (BTDeviceNameConverter::ToUnicodeL( devices->At( 0 )->DeviceName() ) ).AllocL();
                }    
            }        
            
        CleanupStack::PopAndDestroy( devices );
        CleanupStack::PopAndDestroy( devMan );
        }
    return prompt;        
    }

// End of File



