/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies). 
* All rights reserved.
* This component and the accompanying materials are made available
* under the terms of "Eclipse Public License v1.0"
* which accompanies this distribution, and is available
* at the URL "http://www.eclipse.org/legal/epl-v10.html".
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  CSysApCenRepHacSettingObserver implementation
*
*/


// INCLUDE FILES
#include <HWRMDomainCRKeys.h>
#include "SysApCenRepHacSettingObserver.h"
#include "SysApAppUi.h"


// ========================== MEMBER FUNCTIONS ================================

// ----------------------------------------------------------------------------
// CSysApCenRepHacSettingObserver* CSysApCenRepHacSettingObserver::NewL()
// ----------------------------------------------------------------------------

CSysApCenRepHacSettingObserver* CSysApCenRepHacSettingObserver::NewL( CSysApAppUi& aSysApAppUi )
    {       
     //
    CSysApCenRepHacSettingObserver* self = new ( ELeave ) CSysApCenRepHacSettingObserver( aSysApAppUi );
    CleanupStack::PushL( self );
    self->ConstructL();
    CleanupStack::Pop(); //self
    return self;
    }

// ----------------------------------------------------------------------------
// CSysApCenRepHacSettingObserver::CSysApCenRepHacSettingObserver( )
// ----------------------------------------------------------------------------

CSysApCenRepHacSettingObserver::CSysApCenRepHacSettingObserver( CSysApAppUi& aSysApAppUi  )
    : iSysApAppUi( aSysApAppUi )
    {
     //
    }

// ----------------------------------------------------------------------------
// CSysApCenRepHacSettingObserver::~CSysApCenRepHacSettingObserver()
// ----------------------------------------------------------------------------

CSysApCenRepHacSettingObserver::~CSysApCenRepHacSettingObserver()
    {
     //
    if ( iHacSettingHandler )
        {
        iHacSettingHandler->StopListening();
        }
    delete iHacSettingHandler;
    delete iSession;
    }

// ----------------------------------------------------------------------------
// CSysApCenRepHacSettingObserver::ConstructL()
// ----------------------------------------------------------------------------

void CSysApCenRepHacSettingObserver::ConstructL()
    {
     //
    
    TRAPD( err, iSession = CRepository::NewL( KCRUidHWSettings ) );
    
    if ( err == KErrNone)
        {
        iHacSettingHandler = CCenRepNotifyHandler::NewL( *this, 
                                                     *iSession, 
                                                     CCenRepNotifyHandler::EIntKey, 
                                                     KHWSettingsHearingAidMode );
        iHacSettingHandler->StartListeningL();    
        }
    else
        {
         //
        }
    
    }

// ----------------------------------------------------------------------------
// CSysApCenRepHacSettingObserver::HandleNotifyInt()
// ----------------------------------------------------------------------------

void CSysApCenRepHacSettingObserver::HandleNotifyInt( TUint32 /* aId */, TInt /* aNewValue */ )
    {
     //
    
    TRAPD( err, iSysApAppUi.SetHacIndicatorL() );
    
    if ( err )
        {
         //
        }
    }

// ----------------------------------------------------------------------------
// CSysApCenRepHacSettingObserver::HandleNotifyError()
// ----------------------------------------------------------------------------

void CSysApCenRepHacSettingObserver::HandleNotifyError( TUint32 /* aId */, TInt /* error */, CCenRepNotifyHandler* /* aHandler */ )
    {
     //
    }

// ----------------------------------------------------------------------------
// CSysApCenRepHacSettingObserver::HacMode()  
// ----------------------------------------------------------------------------

TInt CSysApCenRepHacSettingObserver::HacMode() const
    {
    TInt hacMode(0);
    
    if ( iSession )
        {
        if ( iSession->Get( KHWSettingsHearingAidMode, hacMode ) != KErrNone )
            {
             //
            }    
        }
        
    return hacMode;
    }
    

// End of File

