/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies). 
* All rights reserved.
* This component and the accompanying materials are made available
* under the terms of "Eclipse Public License v1.0"
* which accompanies this distribution, and is available
* at the URL "http://www.eclipse.org/legal/epl-v10.html".
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  CSysApCenRepLogsObserver implementation.
 *
*/


// INCLUDE FILES
#include "SysApCenRepLogsObserver.h"
#include "SysApAppUi.h"

// ========================== MEMBER FUNCTIONS ================================

// ----------------------------------------------------------------------------
// CSysApCenRepLogsObserver* CSysApCenRepLogsObserver::NewL()
// ----------------------------------------------------------------------------

CSysApCenRepLogsObserver* CSysApCenRepLogsObserver::NewL( CSysApAppUi& aSysApAppUi )
    {       
     //
    CSysApCenRepLogsObserver* self = new ( ELeave ) CSysApCenRepLogsObserver( aSysApAppUi );
    CleanupStack::PushL( self );
    self->ConstructL();
    CleanupStack::Pop( self );
    return self;
    }

// ----------------------------------------------------------------------------
// CSysApCenRepLogsObserver::CSysApCenRepLogsObserver( CSysApAppUi& aSysApAppUi )
// ----------------------------------------------------------------------------

CSysApCenRepLogsObserver::CSysApCenRepLogsObserver( CSysApAppUi& aSysApAppUi  )
    : iSysApAppUi( aSysApAppUi ),
      iUiReady( EFalse ),
      iSimChangedCheckDone( EFalse )
    {
     //
    }

// ----------------------------------------------------------------------------
// CSysApCenRepLogsObserver::~CSysApCenRepLogsObserver()
// ----------------------------------------------------------------------------

CSysApCenRepLogsObserver::~CSysApCenRepLogsObserver()
    {
     //
    if ( iNewMissedCallsHandler )
        {
        iNewMissedCallsHandler->StopListening();
        }
    delete iNewMissedCallsHandler;
    delete iSession;
    }

// ----------------------------------------------------------------------------
// CSysApCenRepLogsObserver::ConstructL()
// ----------------------------------------------------------------------------

void CSysApCenRepLogsObserver::ConstructL()
    {
    TRAPD( err, iSession = CRepository::NewL( KCRUidLogs ) );
     //
    User::LeaveIfError( err );
    iNewMissedCallsHandler = 
        CCenRepNotifyHandler::NewL( *this, 
                                    *iSession, 
                                    CCenRepNotifyHandler::EIntKey, 
                                    KLogsNewMissedCalls );
                                    iNewMissedCallsHandler->StartListeningL();
    
    iSession->Get(KLogsNewMissedCalls, iMissedCallsValue);
    }

// ----------------------------------------------------------------------------
// CSysApCenRepLogsObserver::HandleNotifyInt()
// ----------------------------------------------------------------------------
void CSysApCenRepLogsObserver::HandleNotifyInt( TUint32 /* aId */, TInt aNewValue )
    {
     //
    
    iMissedCallsValue = aNewValue;
    
    TRAPD( err, UpdateMissedCallsIndicatorL() );
        
    if ( err )
        {
         //
        }
    }

// ----------------------------------------------------------------------------
// CSysApCenRepLogsObserver::HandleNotifyError()
// ----------------------------------------------------------------------------
void CSysApCenRepLogsObserver::HandleNotifyError( TUint32 /* aId */, TInt /* error */, CCenRepNotifyHandler* /* aHandler */ )
    {
     //
    }
    
    
// ----------------------------------------------------------------------------
// CSysApCenRepLogsObserver::HandleUiReady()
// ----------------------------------------------------------------------------
void CSysApCenRepLogsObserver::HandleUiReadyL()
    {
     // 
    
    if ( !iUiReady ) // handle only once during device startup
        {
        iUiReady = ETrue;
            
        UpdateMissedCallsIndicatorL();
        }
    }

// ----------------------------------------------------------------------------
// CSysApCenRepLogsObserver::HandleSimChangedCheckDone()
// ----------------------------------------------------------------------------
void CSysApCenRepLogsObserver::HandleSimChangedCheckDoneL()
    {
     //
    
    if ( !iSimChangedCheckDone ) // handle only once during device startup
        {
        iSimChangedCheckDone = ETrue;
            
        UpdateMissedCallsIndicatorL();
        }
    }

// ----------------------------------------------------------------------------
// CSysApCenRepLogsObserver::UpdateMissedCallsIndicatorL()
// ----------------------------------------------------------------------------
void CSysApCenRepLogsObserver::UpdateMissedCallsIndicatorL()
    {
     //
    
    if ( iSimChangedCheckDone && iUiReady )
        {
        TInt newState( iMissedCallsValue > 0 ? EAknIndicatorStateOn : EAknIndicatorStateOff );
        
        iSysApAppUi.SetIndicatorStateL( EAknIndicatorMissedCalls, newState );
        }
    }

// End of File

