/*
* Copyright (c) 2006 Nokia Corporation and/or its subsidiary(-ies). 
* All rights reserved.
* This component and the accompanying materials are made available
* under the terms of "Eclipse Public License v1.0"
* which accompanies this distribution, and is available
* at the URL "http://www.eclipse.org/legal/epl-v10.html".
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:  A Central Repository observer for getting notifications on light
*                settings changes.
*
*/


#include <e32cmn.h>
#include <hwrmlightdomaincrkeys.h>
#include "sysapcenreplightsettingsobserver.h"
#include "SysAp.hrh"
#include "SysApAppUi.h"

// ======== MEMBER FUNCTIONS ========

// ---------------------------------------------------------------------------
// CSysApCenRepLightSettingsObserver::CSysApCenRepLightSettingsObserver
// ---------------------------------------------------------------------------
//
CSysApCenRepLightSettingsObserver::CSysApCenRepLightSettingsObserver( CSysApAppUi& aSysApAppUi )
    : iSysApAppUi( aSysApAppUi ),
      iSession( NULL ),
      iDisplayLightsTimeoutHandler( NULL )
    {
     //
    }

// ---------------------------------------------------------------------------
// CSysApCenRepLightSettingsObserver::ConstructL
// ---------------------------------------------------------------------------
//
void CSysApCenRepLightSettingsObserver::ConstructL()
    {
     //
    
    iSession = CRepository::NewL( KCRUidLightSettings );

    iDisplayLightsTimeoutHandler = CCenRepNotifyHandler::NewL( *this, 
                                                               *iSession, 
                                                               CCenRepNotifyHandler::EIntKey, 
                                                               KDisplayLightsTimeout );
    iDisplayLightsTimeoutHandler->StartListeningL();
    }

// ---------------------------------------------------------------------------
// CSysApCenRepLightSettingsObserver::NewL
// ---------------------------------------------------------------------------
//
CSysApCenRepLightSettingsObserver* CSysApCenRepLightSettingsObserver::NewL( CSysApAppUi& aSysApAppUi )
    {
     //
    CSysApCenRepLightSettingsObserver* self = new( ELeave ) CSysApCenRepLightSettingsObserver( aSysApAppUi);
    CleanupStack::PushL( self );
    self->ConstructL();
    CleanupStack::Pop( self );
    return self;
    }

// ---------------------------------------------------------------------------
// CSysApCenRepLightSettingsObserver::~CSysApCenRepLightSettingsObserver
// ---------------------------------------------------------------------------
//
CSysApCenRepLightSettingsObserver::~CSysApCenRepLightSettingsObserver()
    {
     //
    if ( iDisplayLightsTimeoutHandler )
        { 
        iDisplayLightsTimeoutHandler->StopListening();
        }
    delete iDisplayLightsTimeoutHandler;
    delete iSession;
    }

// ---------------------------------------------------------------------------
// CSysApCenRepLightSettingsObserver::GetLightsTimeout
// ---------------------------------------------------------------------------
//
TInt CSysApCenRepLightSettingsObserver::GetLightsTimeout() const
    {
     //
    
    TInt timeout( 0 );
    
    TInt err = iSession->Get( KDisplayLightsTimeout, timeout );

    if ( err )
        {
         //
        timeout = KDefaultLightsTimeout;
        }
        
    return timeout;
    }

// ---------------------------------------------------------------------------
// From class MCenRepNotifyHandlerCallback.
// CSysApCenRepLightSettingsObserver::HandleNotifyInt
// ---------------------------------------------------------------------------
//
void CSysApCenRepLightSettingsObserver::HandleNotifyInt( TUint32 aId, TInt aNewValue )
    {
     //
    switch ( aId )
        {
        case KDisplayLightsTimeout:
            TRAPD( err, iSysApAppUi.DoLightsTimeoutChangedL( aNewValue ) );
            if ( err )
                {
                 //
                }
            break;
        default:
             //
            break;
        }
    }
  
// ---------------------------------------------------------------------------
// From class MCenRepNotifyHandlerCallback.
// CSysApCenRepLightSettingsObserver::HandleNotifyError
// ---------------------------------------------------------------------------
//
void CSysApCenRepLightSettingsObserver::HandleNotifyError( TUint32 /*aId*/, TInt /*error*/, CCenRepNotifyHandler* /*aHandler*/ )
    {
     //
    }
