/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies). 
* All rights reserved.
* This component and the accompanying materials are made available
* under the terms of "Eclipse Public License v1.0"
* which accompanies this distribution, and is available
* at the URL "http://www.eclipse.org/legal/epl-v10.html".
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description: 
*
*/


// INCLUDE FILES
#include <MediatorDomainUIDs.h>
#include "SysApMediatorObserver.h"
#include "SysApAppUi.h"

// ================= MEMBER FUNCTIONS =======================

// ----------------------------------------------------
// CSysApMediatorObserver::ConstructL()
// ----------------------------------------------------
void CSysApMediatorObserver::ConstructL()
    {
    //

    __ASSERT_ALWAYS( iSysApAppUi, User::Invariant());

#ifndef RD_STARTUP_ANIMATION_CUSTOMIZATION
    //
    iCommandInitiator = CMediatorCommandInitiator::NewL(this);
#endif // RD_STARTUP_ANIMATION_CUSTOMIZATION

    //
    iCommandResponder = CMediatorCommandResponder::NewL(this);

    //
    iEventProvider = CMediatorEventProvider::NewL();

    TInt err = iCommandResponder->RegisterCommand(KMediatorSecondaryDisplayDomain,
                                       SecondaryDisplay::KCatStartup,
                                       SecondaryDisplay::ECmdStartupPhaseSkip,
                                       TVersion(1,0,0),
                                       ECapabilitySwEvent,
                                       KMaximumAnimationTime);

    if ( err != KErrNone )
        {
        // If Startup application has not unregistered command, registration will fail
        //
        }

    err = iEventProvider->RegisterEvent( KMediatorSecondaryDisplayDomain,
                                                SecondaryDisplay::KCatStartup,
                                                SecondaryDisplay::EMsgStartupEvent,
                                                TVersion(1,0,0),
                                                ECapabilitySwEvent );

    if ( err != KErrNone )
        {
        // If Startup application has not unregistered event, registration will fail
        //
        }

    //
    }

// ----------------------------------------------------
// CSysApMediatorObserver::NewL( CSysApAppUi* aStartupAppUi )
// ----------------------------------------------------
CSysApMediatorObserver* CSysApMediatorObserver::NewL( CSysApAppUi* aSysApAppUi )
    {
    //
    CSysApMediatorObserver* self = new (ELeave) CSysApMediatorObserver( aSysApAppUi );

    CleanupStack::PushL( self );
    self->ConstructL();
    CleanupStack::Pop(); // self

    //
    return self;
    }

// ----------------------------------------------------
// CSysApMediatorObserver::CSysApMediatorObserver( CSysApAppUi* aSysApAppUi )
// C++ default constructor can NOT contain any code, that
// might leave.
// ----------------------------------------------------
CSysApMediatorObserver::CSysApMediatorObserver( CSysApAppUi* aSysApAppUi ) :
    iSysApAppUi( aSysApAppUi )
    {
    //
    }

// ----------------------------------------------------
// CSysApMediatorObserver::~CSysApMediatorObserver()
// ----------------------------------------------------
CSysApMediatorObserver::~CSysApMediatorObserver()
    {
    //
#ifndef RD_STARTUP_ANIMATION_CUSTOMIZATION
    delete iCommandInitiator;
#endif // RD_STARTUP_ANIMATION_CUSTOMIZATION
    delete iCommandResponder;
    delete iEventProvider;
    //
    }

#ifndef RD_STARTUP_ANIMATION_CUSTOMIZATION
// ---------------------------------------------------------
// CSysApMediatorObserver::IssueCommand(TInt aCommandId, TInt aData)
// ---------------------------------------------------------
TInt CSysApMediatorObserver::IssueCommand(TInt aCommandId, TInt aData)
    {
    //

    TPckgBuf<TInt> data( aData );
    TInt err = iCommandInitiator->IssueCommand( KMediatorSecondaryDisplayDomain,
                                       SecondaryDisplay::KCatStartup,
                                       aCommandId,
                                       TVersion(1,0,0),
                                       data);

    if ( err != KErrNone )
        {
        //
        }

    //
    return err;
    }

// ---------------------------------------------------------
// SysApMediatorObserver::CommandResponseL(...)
// ---------------------------------------------------------
void CSysApMediatorObserver::CommandResponseL( TUid /*aDomain*/,
                                                 TUid /*aCategory*/,
                                                 TInt aCommandId,
                                                 TInt /*aStatus*/,
                                                 const TDesC8& /*aData*/ )
    {
    //
    if (aCommandId == SecondaryDisplay::ECmdStartupSync)
        {
        // We only issue shutdown animatio sync, so no need to examine response data
        iSysApAppUi->ShutdownAnimationSyncOK();
        }

    //
    }
#endif // RD_STARTUP_ANIMATION_CUSTOMIZATION

// ---------------------------------------------------------
// CSysApMediatorObserver::MediatorCommandL(...)
// ---------------------------------------------------------
void CSysApMediatorObserver::MediatorCommandL( TUid aDomain,
                                                 TUid aCategory,
                                                 TInt aCommandId,
                                                 TVersion /*aVersion*/,
                                                 const TDesC8& /*aData*/ )
    {
    //

    if (aCommandId == SecondaryDisplay::ECmdStartupPhaseSkip)
        {
        //

        // At this point only skip command can be the shutdown animation skip
        iSysApAppUi->SkipShutdownAnimation();

        TPckgBuf<TInt> data( NULL );
        iCommandResponder->IssueResponse( aDomain,
                                          aCategory,
                                          aCommandId,
                                          KErrNone,
                                          data );
        }

    //
    }


// ---------------------------------------------------------
// CSysApMediatorObserver::CancelMediatorCommand(...)
// ---------------------------------------------------------
void CSysApMediatorObserver::CancelMediatorCommand( TUid /*aDomain*/,
                                                      TUid /*aCategory*/,
                                                      TInt /*aCommandId*/ )
    {
    //

    // Do nothing.
    }

// ---------------------------------------------------------
// SysApMediatorObserver::ShutdownAnimationSkipped()
// ---------------------------------------------------------
void CSysApMediatorObserver::ShutdownAnimationSkipped()
    {
    //

    // Raise skip event
    TPckgBuf<TInt> data( SecondaryDisplay::EShutdownAnimSkipped );
    TInt err = iEventProvider->RaiseEvent( KMediatorSecondaryDisplayDomain,
                                       SecondaryDisplay::KCatStartup,
                                       SecondaryDisplay::EMsgStartupEvent,
                                       TVersion(1,0,0),
                                       data);

    if ( err != KErrNone )
        {
        //
        }

    }

#ifndef RD_STARTUP_ANIMATION_CUSTOMIZATION
// ---------------------------------------------------------
// CSysApMediatorObserver::SyncShutdownAnimation()
// ---------------------------------------------------------
TInt CSysApMediatorObserver::SyncShutdownAnimation()
    {
    return IssueCommand( SecondaryDisplay::ECmdStartupSync, SecondaryDisplay::EStartShutdownAnimation );
    }
#endif // RD_STARTUP_ANIMATION_CUSTOMIZATION


//  End of File
