/*
* Copyright (c) 2005-2010 Nokia Corporation and/or its subsidiary(-ies). 
* All rights reserved.
* This component and the accompanying materials are made available
* under the terms of "Eclipse Public License v1.0"
* which accompanies this distribution, and is available
* at the URL "http://www.eclipse.org/legal/epl-v10.html".
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description: 
*     This class is the container class of the CSysApShutdownAnimation.
*     Is used to show welcome animation.
*
*/



#ifdef RD_STARTUP_ANIMATION_CUSTOMIZATION

#include <aknappui.h>
#include <secondarydisplay/SecondaryDisplayStartupAPI.h>
#include <startupdomaincrkeys.h>

#include "SysApShutdownAnimation.h"
#include "SysApShutdownImage.h"
#include "SysAp.hrh"

// ======== MEMBER FUNCTIONS ========

// ---------------------------------------------------------------------------
// CSysApShutdownAnimation::NewL
//
// ---------------------------------------------------------------------------
//
CSysApShutdownAnimation* CSysApShutdownAnimation::NewL( CSysApShutdownImage& aView )
    {

    CSysApShutdownAnimation* self =
        new( ELeave ) CSysApShutdownAnimation( aView );

    return self;
    }


// ---------------------------------------------------------------------------
// CSysApShutdownAnimation::~CSysApShutdownAnimation
//
// ---------------------------------------------------------------------------
//
CSysApShutdownAnimation::~CSysApShutdownAnimation()
    {

    Cancel();

    if ( iCtrl )
        {
        iCtrl->Cancel();
        iView.RemoveComponent();
        delete iCtrl;
        }

    }


// ---------------------------------------------------------------------------
// CSysApShutdownAnimation::HasContent
//
// ---------------------------------------------------------------------------
//
TBool CSysApShutdownAnimation::HasContent() const
    {
    if ( iCtrl ) return iCtrl->HasContent();
    return EFalse;
    }


// ---------------------------------------------------------------------------
// CSysApShutdownAnimation::Play
//
// ---------------------------------------------------------------------------
//
void CSysApShutdownAnimation::Play( const TCallBack& aCallBack )
    {

    if ( !IsActive() )
        {
        if ( iCtrl )
            {
            delete iCtrl;
            iCtrl = NULL;
            }

        iAvkonAppUi->StatusPane()->MakeVisible( EFalse );
        iView.SetRect( iAvkonAppUi->ApplicationRect() );
        iState = ELoading;

        TRAPD( errorCode, iCtrl = CSAnimStartupCtrl::NewL( iView.Rect(), iView ) );
        if ( errorCode == KErrNone )
            {
            iCallBack = aCallBack;

            iCtrl->SetRect( iView.Rect() );

            CSAnimStartupCtrl::TAnimationParams params;
            params.iRepositoryUid = KCRUidStartupConf;
                params.iAnimationPathKey = KShutdownAnimationPath;
                params.iFrameDelayKey = KShutdownAnimationFrameDelay;
                params.iEnableScalingKey = KShutdownAnimationScaling;
                params.iTonePathKey = KShutdownTonePath;
                params.iVolumeKey = KShutdownToneVolume;

            iCtrl->Load(
                params,
                EFalse,
                SecondaryDisplay::EStartShutdownAnimation,
                iStatus );
            SetActive();
            }
        else
            {

            aCallBack.CallBack();
            }
        }

    }


// ---------------------------------------------------------------------------
// CSysApShutdownAnimation::DoCancel
//
// ---------------------------------------------------------------------------
//
void CSysApShutdownAnimation::DoCancel()
    {

    if ( iCtrl ) iCtrl->Cancel();
    iCallBack.CallBack();

    }


// ---------------------------------------------------------------------------
// CSysApShutdownAnimation::RunL
//
// ---------------------------------------------------------------------------
//
void CSysApShutdownAnimation::RunL()
    {

    if ( iState == ELoading )
        {
        iState = EPlaying;
        iView.SetComponent( *iCtrl );
        iCtrl->Start( iStatus );
        SetActive();
        }
    else if ( iState == EPlaying )
        {
        iState = EFinished;
		if ( iCtrl )
			{
			iCtrl->Cancel();
			iView.RemoveComponent();
			delete iCtrl;
			iCtrl = NULL;
			}	
        iCallBack.CallBack();
        }

    }


// ---------------------------------------------------------------------------
// CSysApShutdownAnimation::CSysApShutdownAnimation
//
// ---------------------------------------------------------------------------
//
CSysApShutdownAnimation::CSysApShutdownAnimation( CSysApShutdownImage& aView )
  : CActive( EPriorityStandard ),
    iView( aView ),
    iState( EInitial )
    {

    CActiveScheduler::Add( this );

    }


#else // RD_STARTUP_ANIMATION_CUSTOMIZATION

// INCLUDE FILES
#include <aknappui.h>
#include <aknnotewrappers.h>
#include <barsread.h> //use of TResourceReader
#include <aknbitmapanimation.h>
#include <ConeResLoader.h>
#include "SysApShutdownAnimation.h"
#include <SysApShutdownAnimation.rsg>
#include "SysAp.hrh"
#include "SysApAppUi.h"
#include "SysApShutdownTone.h"

// CONSTANTS
_LIT( KShutdownAnimationResource, "z:sysapshutdownanimation.rsc");

// ================= MEMBER FUNCTIONS =======================

// ---------------------------------------------------------------------------
// CSysApShutdownAnimation::ConstructL()
// Symbian 2nd phase constructor can leave.
// ---------------------------------------------------------------------------
void CSysApShutdownAnimation::ConstructL()
    {

    CreateWindowL();

    iAnimTimer = CPeriodic::NewL( EPriorityNormal );

    iAvkonAppUi->StatusPane()->MakeVisible(EFalse);

    iAnim = CAknBitmapAnimation::NewL();
    iAnim->SetContainerWindowL( *this );
    iAnim->SetScaleModeForAnimationFrames(EAspectRatioPreservedAndUnusedSpaceRemoved);
    TResourceReader rr;
    RConeResourceLoader loader( *iCoeEnv );

    TParse* fp = new(ELeave) TParse();
    fp->Set(KShutdownAnimationResource, &KDC_APP_RESOURCE_DIR, NULL);

    TFileName name( fp->FullName() );
    delete fp;

    TInt fileError = loader.Open( name );
    if ( fileError == KErrNone )
        {
        CleanupClosePushL( loader );
        iCoeEnv->CreateResourceReaderLC(rr, R_SHUTDOWN_ANIM);
        TRAPD(err, iAnim->ConstructFromResourceL( rr ));
        if( err == KErrNone )
            {
            TResourceReader timeReader;
            iCoeEnv->CreateResourceReaderLC(timeReader, R_ANIM_DURATION);
            iShowingTime = timeReader.ReadInt16();

            CleanupStack::PopAndDestroy(); // pop timeReader

            

            if ( iShowingTime > 0 )
                {
                // Initialize shutdown tone. Animation can be shown without any tone, so trap all leaves.
                // To maintain binary compatibility , check resource availability first to avoid panic in case tone is missing.
                TBool isTonePathAvailable(EFalse);
                TRAP(err, isTonePathAvailable = iCoeEnv->IsResourceAvailableL(R_SHUTDOWN_TONE_PATH));
                if ( isTonePathAvailable )
                    {
                    HBufC* tonePath = NULL;
                    TRAP(err, tonePath = iCoeEnv->AllocReadResourceL(R_SHUTDOWN_TONE_PATH));
                    if ( err != KErrNone )
                        {

                        }
                    else
                        {
                        TRAP(err, iShutdownTone = CSysApShutdownTone::NewL(*tonePath, this));
                        if ( err != KErrNone )
                            {

                            }
                        }
                    delete tonePath;
                    }
                else
                    {

                    }
                }
            }
        else
            {
            iShowingTime = 0;
            
            }

        CleanupStack::PopAndDestroy(); //pop rr

        CleanupStack::PopAndDestroy(); //pop loader
        }
    else
        {
        }


    }

// -----------------------------------------------------------------------------
// CSysApShutdownAnimation::NewL
// Two-phased constructor.
// -----------------------------------------------------------------------------
//
CSysApShutdownAnimation* CSysApShutdownAnimation::NewL( CSysApAppUi* aSysApAppUi)
    {

    CSysApShutdownAnimation* self = new (ELeave) CSysApShutdownAnimation( aSysApAppUi );
    CleanupStack::PushL(self);
    self->ConstructL();
    CleanupStack::Pop();
    return self;
    }

// ---------------------------------------------------------
// CSysApShutdownAnimation::CSysApShutdownAnimation()
// ---------------------------------------------------------
CSysApShutdownAnimation::CSysApShutdownAnimation( CSysApAppUi* aSysApAppUi ) :
    iSysApAppUi( aSysApAppUi ),
    iAnimationShowing( EFalse ),
    iAnimationCancelled ( EFalse ),
    iShutdownTone( NULL ),
    iHandleNextKey( ETrue )
    {
    }

// ---------------------------------------------------------------------------
// CSysApShutdownAnimation::StartL()
// ---------------------------------------------------------------------------
void CSysApShutdownAnimation::StartL( TBool iShortPowerKeyPressShutdown )
    {

    ControlEnv()->WsSession().Flush(); // force draw of the context

    SetRect(iAvkonAppUi->ApplicationRect());
    iAnim->SetPosition( TPoint( (iAvkonAppUi->ApplicationRect().Width()/2) - (iAnim->BitmapAnimData()->Size().iWidth/2),
                                (iAvkonAppUi->ApplicationRect().Height()/2) - (iAnim->BitmapAnimData()->Size().iHeight/2) ) );

    ActivateL();

    iReadyForToneSync = ETrue;

    // If the device is switched off with a long power key press, the next event key must be ignored as it is leaked from application class
    if ( !iShortPowerKeyPressShutdown )
        {
        iHandleNextKey = EFalse;
        }

    if ( iShutdownTone )
        {
        // Shutdown tone takes a while to initialize before it can be played, so synchronize here.
        if ( iToneReady )
            {
            // Tone already ready, so continue start
            ContinueStart();
            }
        else
            {
            }
        }
    else
        {
        // No tone, so pretend that sync is done
        ContinueStart();
        }

    }

// ---------------------------------------------------------------------------
// CSysApShutdownAnimation::ContinueStart()
// ---------------------------------------------------------------------------
void CSysApShutdownAnimation::ContinueStart()
    {
    iAnimationShowing = ETrue;

    if ( iShutdownTone )
        {
        iShutdownTone->Play();
        }

    TRAPD(err, iAnim->StartAnimationL());

    if ( err != KErrNone )
        {
        }

    iSysApAppUi->StartAnimTiming();
    }

// ---------------------------------------------------------------------------
// CSysApShutdownAnimation::ToneReady()
// ---------------------------------------------------------------------------
void CSysApShutdownAnimation::ToneReady()
    {
    // If animation has been canceled, do not continue
    if ( !iAnimationCancelled )
        {
        iToneReady = ETrue;

        if ( iReadyForToneSync )
            {
            ContinueStart();
            }
        else
            {
            }
        }
    }

// ---------------------------------------------------------------------------
// CSysApShutdownAnimation::DoDrawingL()
// ---------------------------------------------------------------------------
void CSysApShutdownAnimation::DoDrawingL() const
    {
    }

// ---------------------------------------------------------------------------
// CSysApShutdownAnimation::~CSysApShutdownAnimation()
// ---------------------------------------------------------------------------
CSysApShutdownAnimation::~CSysApShutdownAnimation()
    {

    if( iAnim )
        {
        if( iAnimationShowing )
            {
            iAnim->CancelAnimation();
            }
        }
    delete iAnim;

    if (iBackgroundBitmap)
        {
        delete iBackgroundBitmap;

        }

    if( iAnimTimer )
        {
        iAnimTimer->Cancel();

        }
    delete iAnimTimer;
    delete iShutdownTone;

    }

// ---------------------------------------------------------------------------
// CSysApShutdownAnimation::ComponentControl(TInt aIndex)
// ---------------------------------------------------------------------------
CCoeControl* CSysApShutdownAnimation::ComponentControl(TInt aIndex) const
    {
    switch ( aIndex )
        {
        case 0:
            {
            return iAnim;
            }
        default:
            {
            return NULL;
            }
        }
    }

// ---------------------------------------------------------------------------
// CSysApShutdownAnimation::CountComponentControls()
// ---------------------------------------------------------------------------
TInt CSysApShutdownAnimation::CountComponentControls() const
    {
    return iAnim ? 1 : 0; // return nbr of controls inside this container
    }

// ---------------------------------------------------------------------------
// CSysApShutdownAnimation::Draw(const TRect& aRect) const
// ---------------------------------------------------------------------------
void CSysApShutdownAnimation::Draw(const TRect& ) const
    {
    TRAPD(err, DoDrawingL());
    if (err != KErrNone)
        {
        }
    }

// ---------------------------------------------------------------------------
// CSysApShutdownAnimation::ShowingTime()
// ---------------------------------------------------------------------------
TInt CSysApShutdownAnimation::ShowingTime()
    {
    return iShowingTime;
    }

// ---------------------------------------------------------------------------
// CSysApShutdownAnimation::SizeChanged()
// ---------------------------------------------------------------------------
void CSysApShutdownAnimation::SizeChanged()
    {
    if( iAnim )
        {
        iAnim->SetRect(Rect());
        }
    }

// ---------------------------------------------------------------------------
// CSysApShutdownAnimation::EndAnimation()
// ---------------------------------------------------------------------------
void CSysApShutdownAnimation::EndAnimation()
    {
    if( iAnim && iAnimationShowing && !iAnimationCancelled )
        {
        iAnim->CancelAnimation();
        iAnimationCancelled = ETrue;
        iAnimationShowing = EFalse;
        // Stop shutdown tone
        if ( iShutdownTone )
            {
            iShutdownTone->Stop();
            }

        // Continue shutdown
        iAnimTimer->Start( 1000, 1000, TCallBack( iSysApAppUi->DoStopAnimTiming, iSysApAppUi ) );
        }
    }

// ---------------------------------------------------------------------------
// CSysApShutdownAnimation::HandleControlEventL(...)
// ---------------------------------------------------------------------------
void CSysApShutdownAnimation::HandleControlEventL(
        CCoeControl* ,
        TCoeEvent )
    {
    //pure virtual from MCoeControlObserver
    }

// ---------------------------------------------------------------------------
// CSysApShutdownAnimation::OfferKeyEventL(...)
// ---------------------------------------------------------------------------
TKeyResponse CSysApShutdownAnimation::OfferKeyEventL( const TKeyEvent& aKeyEvent, TEventCode aType )
    {

    // This block is for filtering the key events that are leaked from CSysApAppUi before the animation is active
    // e.g. when device is shutdown with a long press of power key
    if ( !iHandleNextKey )
        {
        if ( aType == EEventKeyDown || aType == EEventKeyUp )
            {
            iHandleNextKey = ETrue;
            }
        }

    if ( aType == EEventKey )
        {
        if ( !iSysApAppUi->ResourcesFreed() )
            {
            TBool isDeviceModeKey = iSysApAppUi->IsDeviceModeKey( aKeyEvent );

            // aType is used for filtering key events that are received when this control is visible
            if( iAnimationShowing && !iAnimationCancelled && !isDeviceModeKey && iHandleNextKey )
                {
                EndAnimation();
                iSysApAppUi->NotifyShutdownAnimationSkip();
                }

            if ( isDeviceModeKey ) // device mode keys are not consumed, so that application class is able to receive them
                {
                return EKeyWasNotConsumed;
                }
            }
        else  // SysAp has relased almost all the allocations, just cancel
            {
            EndAnimation();
            }
        }

    return EKeyWasConsumed;
    }

#endif // RD_STARTUP_ANIMATION_CUSTOMIZATION

//  End of File
